@@ -1,9 +1,9 @@
 package JE::Code;
 
-our $VERSION = '0.019';
+our $VERSION = '0.020';
 
 use strict;
-use warnings;
+use warnings; no warnings 'utf8';
 
 #use Data::Dumper;
 use Exporter 5.57 'import';
@@ -169,7 +169,7 @@ sub add_line_number {
 
 package JE::Code::Statement; # This does not cover expression statements.
 
-our $VERSION = '0.019';
+our $VERSION = '0.020';
 
 use subs qw'_create_vars _eval_term';
 use List::Util 'first';
@@ -605,7 +605,7 @@ sub _create_vars {  # Process var and function declarations
 
 package JE::Code::Expression;
 
-our $VERSION = '0.019';
+our $VERSION = '0.020';
 
 # B::Deparse showed me how to get these values.
 use constant nan => sin 9**9**9;
@@ -1268,7 +1268,7 @@ sub _eval_term {
 
 package JE::Code::Subscript;
 
-our $VERSION = '0.019';
+our $VERSION = '0.020';
 
 sub str_val {
 	my $val = (my $self = shift)->[1];
@@ -1280,7 +1280,7 @@ sub str_val {
 
 package JE::Code::Arguments;
 
-our $VERSION = '0.019';
+our $VERSION = '0.020';
 
 sub list {
 	my $self = shift;
